this.build_blacksmith_event <- this.inherit("scripts/events/event", {
	m = {
		Blacksmith = null,
		Town = null,
		Costs = 1000
	},
	function create()
	{
		this.m.ID = "event.build_blacksmith";
		this.m.Title = "At %townname%";
		this.m.Cooldown = 1.0 * this.World.getTime().SecondsPerDay;
		this.m.Screens.push({
			ID = "A",
			Text = "[img]gfx/ui/events/event_51.png[/img]Do you want build blacksmith?",
			Image = "",
			List = [],
			Characters = [],
			Options = [
				{
					Text = "I accept your choice and will give you money.",
					function getResult( _event )
					{
						return "B";
					}
				}
			],
			function start( _event )
			{
				this.Characters.push(_event.m.Blacksmith.getImagePath());
			}
		});
		this.m.Screens.push({
			ID = "B",
			Text = "[img]gfx/ui/events/event_51.png[/img]All guys say bye-bye!",
			Image = "",
			List = [],
			Characters = [],
			Options = [
				{
					Text = "Thats all",
					function getResult( _event )
					{
						return 0;
					}
				}
			],
			function start( _event )
			{
				this.Characters.push(_event.m.Blacksmith.getImagePath());

				this.World.Assets.addMoney(-_event.m.Costs);
				_event.m.Town.addBuilding(this.new("scripts/entity/world/settlements/buildings/armorsmith_building"));

				_event.m.Blacksmith.getItems().transferToStash(this.World.Assets.getStash());
				this.World.getPlayerRoster().remove(_event.m.Blacksmith);
				_event.m.Blacksmith = null;

				this.List.push({
					id = 10,
					icon = "ui/icons/asset_money.png",
					text = "You spend [color=" + this.Const.UI.Color.NegativeEventValue + "]" + _event.m.Costs + "[/color] Crowns"
				});

				this.List.push({
					id = 10,
					icon = "ui/icons/special.png",
					text = _event.m.Blacksmith.getName() + " left company"
				});

				this.List.push({
					id = 10,
					icon = "ui/icons/buildings/armorsmith.png",
					text = _event.m.Town.getName() + " now has armorsmith"
				});
			}
		});
	}

	function onUpdateScore()
	{
		if (this.World.Assets.getMoney() < this.m.Costs)
		{
			return;
		}

		local towns = this.World.EntityManager.getSettlements();
		local nearTown = false;
		local town;
		local playerTile = this.World.State.getPlayer().getTile();

		foreach( t in towns )
		{
			if (t.getTile().getDistanceTo(playerTile) <= 3 && t.isAlliedWithPlayer())
			{
				nearTown = true;
				town = t;
				break;
			}
		}

		if (!nearTown)
		{
			return;
		}

		local brothers = this.World.getPlayerRoster().getAll();
		local candidates = [];

		foreach( bro in brothers )
		{
			if (bro.getBackground().getID() == "background.apprentice")
			{
				candidates.push(bro);
			}
		}

		if (candidates.len() == 0)
		{
			return;
		}

		this.m.Blacksmith = candidates[this.Math.rand(0, candidates.len() - 1)];
		this.m.Town = town;
		this.m.Score = 500;
	}

	function onPrepare()
	{
	}

	function onPrepareVariables( _vars )
	{
		_vars.push([
			"blacksmith",
			this.m.Blacksmith.getNameOnly()
		]);
		_vars.push([
			"townname",
			this.m.Town.getName()
		]);
	}

	function onClear()
	{
		this.m.Blacksmith = null;
		this.m.Town = null;
	}
});
