this.deadly_poison_secret_event <- this.inherit("scripts/events/event", {
	m = {
		Assassin = null,
		Town = null,
	},
	function create()
	{
		this.m.ID = "event.deadly_poison_secret";
		this.m.Title = "At %townname%";
		this.m.Cooldown = 1.0 * this.World.getTime().SecondsPerDay;
		this.m.Screens.push({
			ID = "A",
			Text = "[img]gfx/ui/events/event_51.png[/img]{You go to check the inventory and find out that poisoned apple\'s disappeared.%SPEECH_ON%Looking for apple? I took it.%SPEECH_OFF%%assassin% comes you up.%SPEECH_ON%I know, I must ask you, but I have some old deals here. Previous life\'s duty - and only apple is not so much. They needed it for the newest poison and luckily I kept in mind all process - it can be useful for us.%SPEECH_OFF%\n\nYou take thought for a moment if he doesn\'t lie it really can help in future. But others shouldn\'t know about this small incident.}",
			Image = "",
			List = [],
			Characters = [],
			Options = [
				{
					Text = "I hope nobody saw you.",
					function getResult( _event )
					{
						return 0;
					}

				}
			],
			function start( _event )
			{
				this.Characters.push(_event.m.Assassin.getImagePath());

				local stash = this.World.Assets.getStash().getItems();
				foreach( i, item in stash )
				{
					if (item != null && item.getID() == "misc.poisoned_apple")
					{
						stash[i] = null;
						this.List.push({
							id = 10,
							icon = "ui/items/" + item.getIcon(),
							text = "You lose " + item.getName()
						});
						break;
					}
				}

				local itemPosion = this.new("scripts/items/accessory/deadly_poison_item");
				this.List.push({
					id = 10,
					icon = "ui/items/" + itemPosion.getIcon(),
					text = "You get recipe of " + itemPosion.getName()
				});

				this.World.Tags.set("DeadlyPoisonSecret", true);
			}
		});
	}

	function onUpdateScore()
	{
		if (this.World.Tags.get("DeadlyPoisonSecret") == true)
		{
			return;
		}

		local stash = this.World.Assets.getStash().getItems();
		local hasItem = false;

		foreach( item in stash )
		{
			if (item != null && item.getID() == "misc.poisoned_apple")
			{
				hasItem = true;
				break;
			}
		}

		if (!hasItem)
		{
			return;
		}

		local towns = this.World.EntityManager.getSettlements();
		local nearTown = false;
		local town;
		local playerTile = this.World.State.getPlayer().getTile();

		foreach( t in towns )
		{
			if (t.getTile().getDistanceTo(playerTile) <= 3 && t.isAlliedWithPlayer())
			{
				nearTown = true;
				town = t;
				break;
			}
		}

		if (!nearTown)
		{
			return;
		}

		local brothers = this.World.getPlayerRoster().getAll();
		local candidates_assasin = [];

		foreach( bro in brothers )
		{
			if (bro.getBackground().getID() == "background.assassin" || bro.getBackground().getID() == "background.killer_on_the_run")
			{
				candidates_assasin.push(bro);
			}
		}

		if (candidates_assasin.len() == 0)
		{
			return;
		}

		this.m.Assassin = candidates_assasin[this.Math.rand(0, candidates_assasin.len() - 1)];
		this.m.Town = town;
		this.m.Score = 500;
	}

	function onPrepare()
	{
	}

	function onPrepareVariables( _vars )
	{
		_vars.push([
			"assassin",
			this.m.Assassin.getNameOnly()
		]);
		_vars.push([
			"townname",
			this.m.Town.getName()
		]);
	}

	function onClear()
	{
		this.m.Assassin = null;
		this.m.Town = null;
	}
});
