this.blessing_oil_coat_effect <- this.inherit("scripts/skills/skill", {
	m = {
		AttacksLeft = 4,
		Damage = 10
	},
	function create()
	{
		this.m.ID = "effects.blessing_oil_coat";
		this.m.Name = "Weapon coated with blessing oil";
		this.m.Icon = "skills/status_effect_blessing_oil.png";
		this.m.IconMini = "status_effect_88_mini";
		this.m.Type = this.Const.SkillType.StatusEffect;
		this.m.IsActive = false;
		this.m.IsStacking = false;
		this.m.IsRemovedAfterBattle = true;
	}

	function getDescription()
	{
		return "This character is using a weapon coated with blessing oil. The next [color=" + this.Const.UI.Color.PositiveValue + "]" + this.m.AttacksLeft + "[/color] hits will deal [color=" + this.Const.UI.Color.NegativeValue + "]" + this.m.Damage + "[/color] extra damage to Undead.";
	}

	function getTooltip()
	{
		return [
			{
				id = 1,
				type = "title",
				text = this.getName()
			},
			{
				id = 2,
				type = "description",
				text = this.getDescription()
			}
		];
	}

	function resetTime()
	{
		if (this.getContainer().getActor().isPlacedOnMap())
		{
			this.spawnIcon("status_effect_88", this._targetEntity.getTile());
		}

		this.m.AttacksLeft = 4;
	}

	function onTargetHit( _skill, _targetEntity, _bodyPart, _damageInflictedHitpoints, _damageInflictedArmor )
	{
		--this.m.AttacksLeft;

		if (this.m.AttacksLeft <= 0)
		{
			this.removeSelf();
		}

		if (!_targetEntity.getTags().has("undead") || _targetEntity.getHitpoints() <= 0)
		{
			return;
		}

		if (!_targetEntity.isAlive())
		{
			return;
		}

		if (!_targetEntity.isHiddenToPlayer())
		{
			if (this.m.SoundOnUse.len() != 0)
			{
				this.Sound.play(this.m.SoundOnUse[this.Math.rand(0, this.m.SoundOnUse.len() - 1)], this.Const.Sound.Volume.RacialEffect * 1.5, _targetEntity.getPos());
			}

			this.Tactical.EventLog.log(this.Const.UI.getColorizedEntityName(_targetEntity) + " suffers from blessing weapon");
		}

		local hitInfo = clone this.Const.Tactical.HitInfo;
		hitInfo.DamageRegular = this.m.Damage;
		hitInfo.DamageDirect = 1.0;
		hitInfo.BodyPart = _bodyPart;
		hitInfo.BodyDamageMult = 1.0;
		hitInfo.FatalityChanceMult = 0.0;
		_targetEntity.onDamageReceived(this.getContainer().getActor(), this, hitInfo);
	}

	function onTargetMissed( _skill, _targetEntity )
	{
		--this.m.AttacksLeft;

		if (this.m.AttacksLeft <= 0)
		{
			this.removeSelf();
		}
	}

});
